/**
* Demonstrate how to configure, start and get the value of a timer
**/
#include "stdafx.h"

/**
* Interrupt routine
**/
static void TimerInterruptRoutine (
	HANDLE	deviceHandle,
	BYTE	fifoOverflow,
	BYTE	interruptMask,
	PWORD	args,
	DWORD	argsCount )
{
	UNREFERENCED_PARAMETER(args);
	UNREFERENCED_PARAMETER(argsCount);
	UNREFERENCED_PARAMETER(deviceHandle);
	UNREFERENCED_PARAMETER(fifoOverflow);
	printf("Interrupt, mask is : %x \n", interruptMask );
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT	returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	DWORD delay = 0;
	
	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will initialize a timer, with interrupt. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* set the board interrupt routine */
	returnValue = i_PCI3001_SetBoardIntRoutine( boardHandle, TimerInterruptRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3001_SetBoardIntRoutine \n", returnValue);
		i_PCI3001_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* get the delay */
	ReadUlong("Delay you want to use, in us (100 to 838860850) ",100,838860850,&readValue);
	delay = readValue;

	/* start the timer */
	returnValue = i_PCI3001_InitAndStartTimer( boardHandle,
		delay,
		APCI3001_ENABLE); /* with interrupt */

	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3001_InitAndStartTimer\n", returnValue);
		i_PCI3001_ResetBoardIntRoutine(boardHandle);
		i_PCI3001_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3001_InitAndStartTimer OK \n");
	}

	/* endless loop */
	for(;;)
	{
		DWORD value = 0;
		returnValue = i_PCI3001_ReadTimerValue(boardHandle, &value);
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3001_ReadTimerValue \n", returnValue);
			i_PCI3001_ResetBoardIntRoutine(boardHandle);
			i_PCI3001_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* display the value */
		printf("Value is : %ld \t \t \r", value );

		/* does the user hit a key ? */
		if ( _kbhit() )
		{
			_getch();
			break;
		}

		Sleep(100);
	}

	/* stop the timer */
	returnValue = i_PCI3001_StopTimer(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3001_StopTimer\n", returnValue);
	}
	else
	{
		printf("i_PCI3001_StopTimer OK \n");
	}

	/* reset the interrupt routine */
	returnValue = i_PCI3001_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3001_ResetBoardIntRoutine \n", returnValue);
	}
	else
	{
		printf("i_PCI3001_ResetBoardIntRoutine OK \n");
	}

	/* close the board */
	returnValue = i_PCI3001_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3001_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3001_CloseBoard OK \n");
	}

	/* exit */
	printf("\n Press any key to exit the sample ...");
	_getch();

	return 0;
}

