program ProjectSequence;

{$APPTYPE CONSOLE}

{$R *.res}

uses
  System.SysUtils,
  PCI3001 in '..\PCI3001.PAS';

Var
    b_NbrOfBoardFound : Byte;
    i_ReturnValue     : Int16;
    h_BoardHandle      : THandle;
    b_Channel : Array[0..4] of Byte;
    b_Gain : Array[0..4] of Byte;
    b_Polar : Array[0..4] of Byte;
    b_InterruptCpt : Byte;

Procedure v_InterruptRoutine   (h_BoardHandle      : THANDLE;
                                b_FIFOOverflow    : Byte;
                                b_InterruptMask  : Byte;
                                var w_Args  : Uint16;
								                dw_ArgsCount : LongWord);cdecl;

  var
    read : LongInt;
    values : Array[0..8] of Uint16;
    i : Integer;
	Begin

  writeln('');
	Writeln('interrupt mask :' +  IntToStr(b_InterruptMask) + ' ArgsCount :'+ IntToStr(dw_ArgsCount));
  writeln('');

	if ( (b_InterruptMask and 4) = 4 ) then
		Writeln('internal sequence fifo overflow ')
	else if ( (b_InterruptMask and 8) = 8 ) then
  begin
		(* get the sequence data *)
		read := l_PCI3001_ReadSequenceData( h_BoardHandle, dw_ArgsCount * sizeof(Uint16), values[0] );
		if ( read < 0 ) then
			Writeln('l_PCI3116_ReadSequenceData error : ' + IntToStr(read))
		else
    begin
			(* display the values *)
			for i := 0 to (dw_ArgsCount-1) do
      begin
        if((i mod 4) = 0 )then
          writeln('');
				Write( '0x' + IntToHex(values[i],4) +' ');
      end;
      writeln('');
		end;
  end;

  b_InterruptCpt := b_InterruptCpt + 1;

	End;



begin
  try

  (* display some useful informations *)
  Writeln('* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * ');
  Writeln('* This sample will initialize an acquisition using 4 channels and no delay. ');
  Writeln('* The sequence is : ');
  Writeln('*   - Channel 0, bipolar, gain 1 ');
  Writeln('*   - Channel 1, bipolar, gain 1 ');
  Writeln('*   - Channel 2, bipolar, gain 1 ');
  Writeln('*   - Channel 3, bipolar, gain 1 ');
  Writeln('* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * ');

  {* Get the number of xPCI-3001 found  *}
  i_ReturnValue := i_PCI3001_GetNumberOfBoards (b_NbrOfBoardFound);

  {* Test the return value *}
  If (i_ReturnValue <> 0) Then
  Begin
    exit;
  end;

  {* Prints the number of boards *}
  Writeln('Nbr of board found:' + IntToStr(b_NbrOfBoardFound));

  {* Get the board handle *}
  i_ReturnValue := i_PCI3001_OpenBoardViaIndex (0,h_BoardHandle);

  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_OpenBoardViaIndex error:' + IntToStr(i_ReturnValue));
    Readln;
    exit;
  end;

  Writeln('i_PCI3001_OpenBoardViaIndex OK');

  (* Install the interrupt routine *)
  i_ReturnValue := i_PCI3001_SetBoardIntRoutine(h_BoardHandle,
                                                @v_InterruptRoutine);
  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_SetBoardIntRoutine error:' + IntToStr(i_ReturnValue));
    i_PCI3001_CloseBoard(h_BoardHandle);
    Readln;
    exit;
  end;
  Writeln('i_PCI3001_SetBoardIntRoutine OK');

  i_ReturnValue:= i_PCI3001_SetAnalogInputMode(h_BoardHandle,1); // Single-Ended
  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_SetAnalogInputMode error:' + IntToStr(i_ReturnValue));
    i_PCI3001_ResetBoardIntRoutine(h_BoardHandle);
    i_PCI3001_CloseBoard(h_BoardHandle);
    Readln;
    exit;
  end;

  Writeln('i_PCI3001_SetAnalogInputMode OK');

  (* initialize the 4 channels of the acquisition *)
  b_Channel[0] := 0;
  b_Channel[1] := 1;
  b_Channel[2] := 2;
  b_Channel[3] := 3;

  b_Gain[0] := 0;//GAIN 1
  b_Gain[1] := 0;
  b_Gain[2] := 0;
  b_Gain[3] := 0;

  b_Polar[0] := 0;// BIPOLAR
  b_Polar[1] := 0;
  b_Polar[2] := 0;
  b_Polar[3] := 0;

  b_InterruptCpt := 0;

  (* start the analog input sequence acquisition *)
  i_ReturnValue := i_PCI3001_InitAndStartAnalogInputSequence(h_BoardHandle,
                                                             10,					// number of sequences to acquire (0 for continuous)
                                                             2,		      // number of sequences in one interrupt
                                                             4,					// 4 channels
                                                             b_Channel[0],
                                                             b_Gain[0],
                                                             b_Polar[0],
                                                             1000,      //1000us
                                                             30,				// simple mode : no delay
                                                             0,				// no delay
                                                             0);		  // no hardware trigger

 if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_InitAndStartAnalogInputSequence error:' + IntToStr(i_ReturnValue));
    i_PCI3001_ResetBoardIntRoutine(h_BoardHandle);
    i_PCI3001_CloseBoard(h_BoardHandle);
    Readln;
    exit;
  end;

  Writeln('i_PCI3001_InitAndStartAnalogInputSequence OK');

  (*
  * endless loop
  * See the InterruptRoutine in the beginning of this file to see how the messages are displayed
  *)
  while(b_InterruptCpt < 5) do;

  (* stop the acquisition *)
  i_ReturnValue := i_PCI3001_StopAnalogAcquisition( h_BoardHandle );
  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_StopAnalogAcquisition error:' + IntToStr(i_ReturnValue));
  end
  else
    Writeln('i_PCI3001_StopAnalogAcquisition OK');


  i_ReturnValue := i_PCI3001_ResetBoardIntRoutine(h_BoardHandle);
  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_ResetBoardIntRoutine error:' + IntToStr(i_ReturnValue));
  end
  else
    Writeln('i_PCI3001_ResetBoardIntRoutine OK');



  i_ReturnValue := i_PCI3001_CloseBoard(h_BoardHandle);
  if (i_ReturnValue <> 0) then
  begin
    Writeln('i_PCI3001_CloseBoard error:' + IntToStr(i_ReturnValue));
  end
  else
    Writeln('i_PCI3001_CloseBoard OK');



  Writeln('Press enter to escape the sample...');
  Readln;






  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
