using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3001_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            byte mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_SetAnalogInputMode(IntPtr deviceHandle, byte inputMode);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_GetAnalogInputMode(IntPtr deviceHandle, ref byte inputMode);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Read1AnalogInput(IntPtr deviceHandle,
            byte channel, 
            byte gain, 
            byte polarity, 
            UInt16 convertingTime, 
            byte interruptFlag, 
            ref UInt16 channelValue);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_ReadMoreAnalogInputs(IntPtr deviceHandle,
            byte channelCount, 
            byte[] channelArray, 
            byte[] gainArray, 
            byte[] polarityArray, 
            UInt16 convertingTime, 
            byte interruptFlag,
            ushort[] channelValues);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_InitAndStartAnalogInputSequence(IntPtr deviceHandle,
            uint sequenceCount, 
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            short convertingTime,
            byte delayMode,
            uint delayTime,
            byte extTriggerMode);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_StopAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern int l_PCI3001_ReadSequenceData(IntPtr deviceHandle, uint size, ushort[] data);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Set4DigitalOutputsOn(IntPtr deviceHandle, byte mask);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Set4DigitalOutputsOff(IntPtr deviceHandle, byte mask);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputsStatus);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_InitAndStartTimer(IntPtr deviceHandle, UInt32 delayValue, byte interruptFlag);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_StopTimer(IntPtr deviceHandle);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_ReadTimerValue(IntPtr deviceHandle, ref UInt32 readValue);

        [DllImport("PCI3001.dll")]
        public static extern short i_PCI3001_WriteTimerValue(IntPtr deviceHandle, UInt32 writeValue);
    }
}

